@@ -42,7 +42,12 @@ setMethod("as.VCorpus", "partition_bundle", function(x) as(x, "VCorpus") )
 setAs(from = "partition_bundle", to = "VCorpus", def = function(from){
   s_attr_lengths <- sapply(
     s_attributes(from@objects[[1]]@corpus),
-    function(s_attr) cl_attribute_size(corpus = from@objects[[1]]@corpus, attribute = s_attr, attribute_type = "s", registry = registry())
+    function(s_attr)
+      cl_attribute_size(
+        corpus = from@objects[[1]]@corpus,
+        registry = from@objects[[1]]@registry_dir,
+        attribute = s_attr, attribute_type = "s"
+      )
   )
   
   if (length(unique(s_attr_lengths)) == 1L){
