@@ -4,7 +4,8 @@ setAs(from = "partition", to = "subcorpus", def = function(from){
     
     # slots inherited from class 'corpus'
     corpus = from@corpus,
-    data_dir = registry_get_home(from@corpus),
+    registry_dir = from@registry_dir,
+    data_dir = from@data_dir,
     type = if (grepl("^.*?_partition$", class(from))){
         as.vector(gsub("^(.*?)_partition$", "\\1", class(from)))
       } else {
@@ -15,6 +16,8 @@ setAs(from = "partition", to = "subcorpus", def = function(from){
     # slots inherited from class 'regions'
     cpos = from@cpos,
     size = from@size,
+    info_file = from@info_file,
+    template = from@template,
     
     name = unname(from@name),
 
@@ -33,12 +36,6 @@ setAs(from = "partition", to = "subcorpus", def = function(from){
   y
 })
 
-setAs(from = "subcorpus", to = "plpr_subcorpus", def = function(from){
-  y <- new("plpr_subcorpus")
-  for (x in slotNames(y)) slot(y, x) <- slot(from, x)
-  y
-})
-
 
 setAs(from = "plpr_partition", to = "plpr_subcorpus", def = function(from){
   as(as(from, "subcorpus"), "plpr_subcorpus")
@@ -49,9 +46,3 @@ setAs(from = "press_partition", to = "press_subcorpus", def = function(from){
   as(as(from, "subcorpus"), "press_subcorpus")
 })
 
-
-setAs(from = "subcorpus", to = "press_subcorpus", def = function(from){
-  y <- new("press_subcorpus")
-  for (x in slotNames(y)) slot(y, x) <- slot(from, x)
-  y
-})
\ No newline at end of file
