@@ -8,11 +8,17 @@ setGeneric("dispersion", function(.Object, ...){standardGeneric("dispersion")})
 
 #' Dispersion of a query or multiple queries.
 #' 
-#' The method returns counts (optionally frequencies) of a query or a multiple
-#' queries in subcorpora defined by one or two s-attributes.
+#' The method returns a \code{data.table} with the number of matches of a query
+#' or multiple queries (optionally frequencies) in a corpus or subcorpus as
+#' partitioned by one or two s-attributes.
 #' 
-#' @param .Object A \code{partition} object or a corpus provided by a character
-#'   string.
+#' @details Augmenting the \code{data.table} with zeros for subcorpora that do
+#'   not yield query matches (argument \code{fill} = \code{TRUE}) may require
+#'   adding many new columns. A respective warning issued by the
+#'   \code{data.table} package is supplemented an additional explanatory note
+#'   of the polmineR package.
+#' @param .Object A \code{corpus}, \code{subcorpus} or \code{partition} object
+#'   or a corpus provided by a character string.
 #' @param query A \code{character} vector stating one or multiple queries.
 #' @param s_attribute A \code{character} vector (length 1 or 2) providing s-attributes.
 #' @param p_attribute Length one \code{character} vector, the p-attribute that
@@ -21,11 +27,15 @@ setGeneric("dispersion", function(.Object, ...){standardGeneric("dispersion")})
 #'   function that is passed in, the function will be applied to the query to
 #'   guess whether query is a CQP query
 #' @param freq A \code{logical} value, whether to calculate normalized frequencies.
+#' @param fill A \code{logical} value, whether to report zero matches. Defaults
+#'   to \code{TRUE}. But note that if there are few matches and many values of
+#'   the s-attribute(s), the resulting data structure is sparse and potentially
+#'   bloated.
 #' @param mc A \code{logical} value, whether to use multicore.
 #' @param verbose A \code{logical} value, whether to be verbose.
 #' @param progress A \code{logical} value, whether to show progress.
 #' @param ... Further parameters.
-#' @return depends on the input, as this is a wrapper function
+#' @return A \code{data.table}.
 #' @seealso The worker behind the \code{dispersion}-method is the \code{hits}-method.
 #' @exportMethod dispersion
 #' @examples
@@ -47,50 +57,64 @@ setGeneric("dispersion", function(.Object, ...){standardGeneric("dispersion")})
 #' @rdname dispersion-method
 #' @aliases dispersion,slice-method
 #' @name dispersion
-setMethod("dispersion", "slice", function(.Object, query, s_attribute, cqp = FALSE, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, mc = FALSE, progress = FALSE, verbose = FALSE, ...){
+setMethod("dispersion", "slice", function(.Object, query, s_attribute, cqp = FALSE, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, fill = TRUE, mc = FALSE, progress = FALSE, verbose = FALSE, ...){
   dot_list <- list(...)
-  if ("sAttribute" %in% names(dot_list)) s_attribute <- dot_list[["sAttribute"]]
-  if ("pAttribute" %in% names(dot_list)) p_attribute <- dot_list[["pAttribute"]]
+  if ("sAttribute" %in% names(dot_list)){
+    warning("The `sAttribute` argument of `dispersion()` is deprecated. Please use argument `s_attribute` instead.")
+    s_attribute <- dot_list[["sAttribute"]]
+  }
+  if ("pAttribute" %in% names(dot_list)){
+    warning("The `pAttribute` argument of `dispersion()` is deprecated. Please use argument `p_attribute` instead.")
+    p_attribute <- dot_list[["pAttribute"]]
+  }
   
   h <- hits(
     .Object = .Object, query = query, cqp = cqp,
     s_attribute = s_attribute, p_attribute = p_attribute, freq = freq,
     mc = mc, verbose = verbose, progress = progress
   )
-  dispersion(h, s_attribute = s_attribute, source = .Object, freq = freq)
+  dispersion(h, s_attribute = s_attribute, source = .Object, freq = freq, fill = fill)
 })
 
 #' @rdname dispersion-method
 setMethod("dispersion", "partition", function(
   .Object, query, s_attribute, cqp = FALSE, p_attribute = getOption("polmineR.p_attribute"),
-  freq = FALSE, mc = FALSE, progress = TRUE, verbose = FALSE, ...){
+  freq = FALSE, fill = TRUE, mc = FALSE, progress = TRUE, verbose = FALSE, ...){
   callNextMethod()
 })
 
 #' @rdname dispersion-method
 setMethod("dispersion", "subcorpus", function(
   .Object, query, s_attribute, cqp = FALSE, p_attribute = getOption("polmineR.p_attribute"),
-  freq = FALSE, mc = FALSE, progress = TRUE, verbose = FALSE, ...){
+  freq = FALSE, fill = FALSE, mc = FALSE, progress = TRUE, verbose = FALSE, ...){
   callNextMethod()
 })
 
 
 #' @rdname dispersion-method
-setMethod("dispersion", "corpus", function(.Object, query, s_attribute, cqp = is.cqp, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, mc = FALSE, progress = FALSE, verbose = FALSE, ...){
+setMethod("dispersion", "corpus", function(.Object, query, s_attribute, cqp = is.cqp, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, fill = TRUE, mc = FALSE, progress = FALSE, verbose = FALSE, ...){
   dot_list <- list(...)
-  if ("sAttribute" %in% names(dot_list)) s_attribute <- dot_list[["sAttribute"]]
-  if ("pAttribute" %in% names(dot_list)) p_attribute <- dot_list[["pAttribute"]]
-  
+  if ("sAttribute" %in% names(dot_list)){
+    warning("The `sAttribute` argument of `dispersion()` is deprecated. Please use argument `s_attribute` instead.")
+    s_attribute <- dot_list[["sAttribute"]]
+  }
+  if ("pAttribute" %in% names(dot_list)){
+    warning("The `pAttribute` argument of `dispersion()` is deprecated. Please use argument `p_attribute` instead.")
+    p_attribute <- dot_list[["pAttribute"]]
+  }
+
   h <- hits(
-    .Object, query = query, cqp = cqp, s_attribute = s_attribute, p_attribute = p_attribute, freq = freq,
+    .Object, query = query, cqp = cqp,
+    s_attribute = s_attribute, p_attribute = p_attribute, 
+    freq = freq,
     mc = mc, verbose = verbose, progress = progress
   )
-  dispersion(h, s_attribute = s_attribute, source = .Object, freq = freq)
+  dispersion(h, s_attribute = s_attribute, source = .Object, freq = freq, fill = fill)
 })
 
 
 #' @rdname dispersion-method
-setMethod("dispersion", "character", function(.Object, query, s_attribute, cqp = is.cqp, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, mc = FALSE, progress = TRUE, verbose = TRUE, ...){
+setMethod("dispersion", "character", function(.Object, query, s_attribute, cqp = is.cqp, p_attribute = getOption("polmineR.p_attribute"), freq = FALSE, fill = TRUE, mc = FALSE, progress = TRUE, verbose = TRUE, ...){
   dispersion(
     .Object = corpus(.Object),
     query = query,
@@ -98,6 +122,7 @@ setMethod("dispersion", "character", function(.Object, query, s_attribute, cqp =
     cqp = cqp,
     p_attribute = p_attribute,
     freq = freq,
+    fill = fill,
     mc = mc,
     progress = progress,
     verbose = verbose,
@@ -106,21 +131,28 @@ setMethod("dispersion", "character", function(.Object, query, s_attribute, cqp =
 })
 
 
+#' @importFrom data.table truelength
 #' @rdname dispersion-method
 #' @param source The source of the evaluation the hits reported in
 #'   \code{.Object} are based on, a \code{corpus}, \code{subcorpus} or
 #'   \code{partition} object.
-setMethod("dispersion", "hits", function(.Object, source, s_attribute, freq = FALSE, verbose = TRUE, ...){
+setMethod("dispersion", "hits", function(.Object, source, s_attribute, freq = FALSE, fill = TRUE, verbose = TRUE, ...){
   
   dot_list <- list(...)
-  if ("sAttribute" %in% names(dot_list)) s_attribute <- dot_list[["sAttribute"]]
-  if ("pAttribute" %in% names(dot_list)) p_attribute <- dot_list[["pAttribute"]]
-  
+  if ("sAttribute" %in% names(dot_list)){
+    warning("The `sAttribute` argument of `dispersion()` is deprecated. Please use argument `s_attribute` instead.")
+    s_attribute <- dot_list[["sAttribute"]]
+  }
+  if ("pAttribute" %in% names(dot_list)){
+    warning("The `pAttribute` argument of `dispersion()` is deprecated. Please use argument `p_attribute` instead.")
+    p_attribute <- dot_list[["pAttribute"]]
+  }
+
   if (!length(s_attribute) %in% c(1L, 2L))
     stop(sprintf("Number of s-attributes is %d but only 1 or 2 s-attributes are allowed.", length(s_attribute)))
   
   if (length(.Object@query) > 1L){
-    if (!freq){
+    if (isFALSE(freq)){
       .Object@stat <- .Object@stat[, {sum(.SD[["count"]])}, by = s_attribute][, "query" := paste(.Object@query, collapse = "//")]
       setnames(.Object@stat, old = "V1", new = "count")
       setcolorder(.Object@stat, neworder = c("query", "count", s_attribute))
@@ -135,7 +167,7 @@ setMethod("dispersion", "hits", function(.Object, source, s_attribute, freq = FA
   if (length(s_attribute) == 1L){
     dt <- .Object@stat
     # ensure that zero matches are reported for all values of the s-attribute
-    if (!freq){
+    if (isTRUE(fill)){
       s_attr_values <- s_attributes(source, s_attribute = s_attribute, unique = TRUE)
       dt <- dt[do.call(data.table, setNames(list(s_attr_values, s_attribute), c(s_attribute, "key"))), on = s_attribute]
       dt[, "count" := ifelse(is.na(dt[["count"]]), 0L, dt[["count"]])]
@@ -153,17 +185,28 @@ setMethod("dispersion", "hits", function(.Object, source, s_attribute, freq = FA
       .Object@stat, formula(paste(s_attribute, collapse = "~")),
       value.var = if (freq) "freq" else "count", fun.aggregate = sum, fill = 0L
     )
-    if (!freq){
+    
+    if (isTRUE(fill)){
       s_attr_values <- s_attributes(source, s_attribute = s_attribute[1], unique = TRUE)
       dt <- dt[do.call(data.table, setNames(list(s_attr_values, s_attribute[1]), c(s_attribute[1], "key"))), on = s_attribute[1]]
       dt[is.na(dt)] <- 0L
       
       s_attr_values <- s_attributes(source, s_attribute = s_attribute[2], unique = TRUE)
       s_attr_values_missing <- s_attr_values[!s_attr_values %in% colnames(dt)]
-      for (s_attr in s_attr_values_missing) dt[, (s_attr) := 0L]
+      length_old <- length(dt)
+      dt[, (s_attr_values_missing) := 0L]
+      if (truelength(dt) > length_old + 10000L){
+        warning(
+          "Supplementary explanatory note on the data.table warning (issued by polmineR): ",
+          "Augmenting the data.table with zeros for all corpus subsets that do not evoke query matches ", 
+          sprintf("requires adding %d columns (with zeroes). This is more than data.table does without issuing ", length(s_attr_values_missing)),
+          "the warning you see."
+        )
+      }
     }
+    
   } else {
-    warning("length(s_attribute) needs to be 1 or 2")
+    warning("length(s_attribute) required to be either 1 or 2")
   }
   dt
 })
