@@ -8,7 +8,7 @@ test_that(
   {
     p <- partition("REUTERS", id = "127", regex = TRUE)
     y <- as.markdown(p)
-    if (.Platform$OS.type == "windows") y <- iconv(y, from = localeToCharset()[1], to = "UTF-8")
+    if (.Platform$OS.type == "windows") y <- iconv(y, from = encoding(), to = "UTF-8")
     
     refdoc <- system.file(package = "polmineR", "fulltext", "reuters_as_markdown.txt")
     benchmark <- readLines(refdoc, warn = FALSE, encoding = "UTF-8")
@@ -38,7 +38,7 @@ test_that(
       as.speeches(s_attribute_name = "speaker", gap = 100) %>%
       .[[2]]
     y <- as.markdown(m)
-    if (.Platform$OS.type == "windows") y <- iconv(y, from = localeToCharset()[1], to = "UTF-8")
+    if (.Platform$OS.type == "windows") y <- iconv(y, from = encoding(), to = "UTF-8")
     
     refdoc <- system.file(package = "polmineR", "fulltext", "merkel_as_markdown.txt")
     benchmark <- readLines(refdoc, warn = FALSE, encoding = "UTF-8")
